package ucot.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JOptionPane;

import ucot.Messages;
import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.exceptions.FileFormatNotSupportedException;
import ucot.exceptions.UseCaseException;
import ucot.heuristic.AbbottsHeuristic;
import ucot.heuristic.DummyHeuristic;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.InputInterface;
import ucot.input.ProcessMlInputAdapter;
import ucot.input.SimpleInputAdapter;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.output.GXLOutput;
import ucot.output.OutputInterface;
import ucot.parser.DummyParser;
import ucot.parser.ParsedData;
import ucot.parser.ParserInterface;
import ucot.parser.SimpleInputParser;
import ucot.parser.StanfordAdapter;
import ucot.ui.DummyUI;
import ucot.ui.ProgressBarInterface;
import ucot.ui.UIInterface;
import ucot.ui.gui.GUIUtils;
import ucot.ui.gui.GraphicalUI;
import ucot.utils.PropertiesTools;
import ucot.utils.StringTools;

/**
 * This is the implementation of core of the program,
 * the controller of the program.
 * 
 * @author UCOT
 */
public class Core extends ModuleProperties implements ControlInterface {
	
	public static final String FILEFORMAT_NOT_SUPPORTED_ERROR =
		Messages.getString("FILEFORMAT_NOT_SUPPORTED");
	public static final String CANNOT_LOAD_FILES_ERROR =
		Messages.getString("CANNOT_LOAD_FILES");
	public static final String CANNOT_ADD_SELECTED_USE_CASE_ERROR = 
	    Messages.getString("Core.CANNOT_ADD_SELECTED_USE_CASE_ERROR");
	public static final String FILE_NOT_SAVED_ERROR =
		Messages.getString("Core.FILE_NOT_SAVED_ERROR");
	public static final String FILE_NOT_WRITABLE_ERROR =
		Messages.getString("FILE_NOT_WRITABLE");
	public static final String FILE_NOT_FOUND =
		Messages.getString("FILE_NOT_FOUND");
	
	public static String getFileNotFoundMessage(URL file) {
		return Messages.getFormatedMessage("FILE_NOT_FOUND",file.getFile());
	}
	
	private InputCollection inputs = new InputCollection();
	private UseCaseCollection useCases = new UseCaseCollection();
	private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();
	private Vector<HeuristicInterface> heuristics = new Vector<HeuristicInterface>();
	private Vector<ParserInterface> parsers = new Vector<ParserInterface>();
	private Vector<OutputInterface> outputs = new Vector<OutputInterface>();
	private UIInterface userInterface = new DummyUI();
	private ParserInterface currentParser = new DummyParser();
	private HeuristicInterface currentHeuristic = new DummyHeuristic();
	private ProgressBarInterface progressBar = new DummyProgressBar();
	private Logger logger = Logger.getLogger("ucot.core");
	
	protected static Vector<Runnable> parsingThreads = new Vector<Runnable>();
	
	/**
	 * Default constructor for UCOT core component. This initializes
	 * the core component by loading all available modules.
	 */
	public Core() {
		// Set up simple logging system.
		ConsoleHandler consoleHandler = new ConsoleHandler();
		consoleHandler.setLevel(Level.ALL);
		Logger.getLogger("ucot").addHandler(consoleHandler);
		Logger.getLogger("ucot").setLevel(Level.ALL);
		Logger.getLogger("ucot").setUseParentHandlers(false);
		
		analyzeModel.addObserver(new AnalyzeModelLogger());
		
		// Inform user that program is starting.
		System.out.println(Messages.getString("Core.LAUNCHING"));
		

		inputs.add(new SimpleInputAdapter());
		inputs.add(new ProcessMlInputAdapter());
		
		outputs.add(new GXLOutput());
		
		heuristics.add(new AbbottsHeuristic());
		parsers.add(new StanfordAdapter());
        parsers.add(new SimpleInputParser());
        
        
        userInterface = new GraphicalUI(this);
        
        try {
			loadProperties();
		} catch (IOException e) {
			/*
			userInterface.printError(Messages.formatMessage(FILE_NOT_FOUND,
					PropertiesTools.propertiesURL));
					*/
			logger.finest(StringTools.exceptionStackTrace(e));
		}
        
        try {
        	applyProperties();
        } catch (BadPropertyValueException e) {
        	logger.severe(Messages.formatMessage(
        			Messages.getString("Core.CONFIGURATION_FILE_CORRUPTED_ERROR"),
        			PropertiesTools.propertiesURL));
        }
        logger.info(Messages.getString("Core.INITIALIZED"));        
	}
	
	/**
	 * Main method that starts the UCOT core.
	 * @param args Command line arguments.
	 */
	public static void main(String args[]){
		new Core();
	}
	
	
	// ------------ CONTROL INTERFACE -------------
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#shutdown()
	 */
	public void shutdown() {
		try {
			saveProperties();
		} catch (IOException e) {
			userInterface.printError(Messages.formatMessage(FILE_NOT_WRITABLE_ERROR,
					PropertiesTools.propertiesURL));
			logger.warning(StringTools.exceptionStackTrace(e));
		}
		
		// Get rid of use cases, inputs, and such.
		inputs = null;
		useCases = null;
		analyzeModel = null;
		heuristics = null;
		parsers = null;
		outputs = null;		
		currentParser = null;
		currentHeuristic = null;
		
		// TODO Remember to comment out the following line when
		// running test units.
		System.exit(0);
	}

	/**
	 * @see ucot.core.ControlInterface#loadUseCases(java.net.URL)
	 */
	public void loadUseCases(URL url) {    
		
		// see if file has already been loaded into system.
		// if it has ask from user if he/she wants to 
		// refresh it.
		List<URL> urls = this.useCases.getURLs();
		if ( urls.contains(url) ) {
			//TODO localize me
			int returnValue = Integer.parseInt(GUIUtils.questionDialog(null, "File is already loaded. \nDo you want to refresh file?", "File already loaded").toString());

			// if user wanted to refresh file remove use cases
			// from that file and continue executing method. 
			// Else we just leave from method.
			if (returnValue == JOptionPane.YES_OPTION) {				
				this.useCases.removeFromUrl(url);
			} else {
				return;
			}
		}
		
        try {
        	InputInterface input = inputs.getInputForUrl(url);
        	useCases.merge(input.read(url));
        	userInterface.useCasesLoaded();
        } catch (UseCaseException e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools.exceptionStackTrace(e));
        } catch (FileFormatNotSupportedException e) {
        	userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools.exceptionStackTrace(e));
        } catch (FileNotFoundException e) {
        	userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.finest(StringTools.exceptionStackTrace(e));
        } catch (Exception e) {
            userInterface.printError(e.getMessage());
            logger.warning(e.getMessage());
            logger.warning(StringTools.exceptionStackTrace(e));
        }  
    }

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#loadAnalyzeModel(java.net.URL)
	 */
	public void loadAnalyzeModel(URL url) throws IOException {
		if ( url == null ) return;
		
		// Make sure the file exists.
		File file = new File(url.getFile());
		if ( !file.exists() ) {
			throw new FileNotFoundException(Messages.formatMessage(
					Messages.getString("FILE_NOT_FOUND"), url.getFile()));
		}
		
		try {
			// Initialize FileInputStream and ObjectInputStream.
			URLConnection connection = url.openConnection();
			connection.connect();
			InputStream urlInputStream = null;
			ObjectInputStream objectInputStream =  null;
			try {
				urlInputStream = connection.getInputStream();
				objectInputStream = new ObjectInputStream(urlInputStream);
				// Load and typecast analyze model.
				Object loadedModel = objectInputStream.readObject();
				if ( loadedModel instanceof ObjectAnalyzeModel ) {
					analyzeModel = (ObjectAnalyzeModel)loadedModel;
				} else {
					throw new FileFormatNotSupportedException(FILEFORMAT_NOT_SUPPORTED_ERROR);
				}
			} finally {
				if ( objectInputStream != null ) objectInputStream.close();
				if ( urlInputStream != null ) urlInputStream.close();
			}
		} catch (FileFormatNotSupportedException e) {
			throw e;
		} catch (Exception e) {
			userInterface.printError(Messages.formatMessage(CANNOT_LOAD_FILES_ERROR,
					url.getFile()));
			logger.warning(e.getMessage());
			logger.warning(StringTools.exceptionStackTrace(e));
		}
		
        userInterface.analyzeModelLoaded();
	}
	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getUseCaseCollection()
	 */
	public UseCaseCollection getUseCaseCollection() {		
		return useCases;
	}
	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#clearAnalyzeModel()
	 */
	public void clearAnalyzeModel() {
		analyzeModel.getEditor().clearModel();
		useCases.markAllUnanalyzed();
	}

	/**
	 * This method runs the parser and heuristic in a single thread.
	 * @param runnable Runnable where this thread is running.
	 * @param useCases Use cases to be parsed.
	 * @param parser Parser to use.
	 * @param heuristic Heuristic to use.
	 */
	private void runParserAndHeuristic(Runnable runnable, Vector<UseCase> useCases,
			ParserInterface parser,	HeuristicInterface heuristic) {
		
		// Wait for own turn to execute (older threads should execute first).
		while ( parsingThreads.firstElement() != runnable ) {
			try {
				synchronized (this) {
					wait();
				}
			} catch (InterruptedException e) {
			} catch (Exception e) {
				logger.warning(StringTools.exceptionStackTrace(e));
			}
		}
		
		// Initialize progress bar.
		progressBar.setValue(0);
		progressBar.setMaximum(useCases.size() * 100);
		progressBar.setString(Messages.getString("ProgressBar.INITIALIZING"));
		progressBar.setVisible(true);

        AnalyzeModel temp = null;
		
		for ( UseCase useCase : useCases ) {
		    
            temp = new ObjectAnalyzeModel();
            
            try {
				progressBar.setValue(progressBar.getValue() + 5);
				progressBar.setString(Messages.getString("ProgressBar.PARSING"));

				ParsedData data = parser.parse(useCase);
				progressBar.setValue(progressBar.getValue() + 55);
				progressBar.setString(Messages.getString("ProgressBar.RUNNING_HEURISTIC"));

				heuristic.doHeuristic(data,temp);
				
				progressBar.setValue(progressBar.getValue() + 25);
				progressBar.setString(Messages.getString("ProgressBar.MERGING"));

				useCase.setAnalyzeModel(temp);
				analyzeModel.getEditor().merge(temp);

				progressBar.setValue(progressBar.getValue() + 15);
				progressBar.setString(Messages.getString("ProgressBar.DONE"));
					
				useCase.setUseCaseAnalyzed();
		        userInterface.useCaseAdded();
			} catch(Exception e) {
				userInterface.printError(CANNOT_ADD_SELECTED_USE_CASE_ERROR);
				logger.warning(StringTools.exceptionStackTrace(e));
			}
		}
		
		// Hide progress bar.
		progressBar.setVisible(false);
		progressBar.setValue(0);

		// Remove this parsing thread from execution list and notify others.
		synchronized (parsingThreads) {
			parsingThreads.remove(parsingThreads.firstElement());
		}
		synchronized (this) {
			notifyAll();
		}
	}
	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#addToAnalyzeModel(ucot.input.UseCase, ucot.parser.ParserInterface, ucot.heuristic.HeuristicInterface)
	 */
	public void addToAnalyzeModel(final Vector<UseCase> useCases,
			final ParserInterface parser,
			final HeuristicInterface heuristic) {
		if ( heuristic == null || parser == null ) return;
		if ( parsers.indexOf(parser) < 0 ) return;
		if ( heuristics.indexOf(heuristic) < 0 ) return;
		
		// Initialize progress bar.
		progressBar = userInterface.getProgressBar();
		
		// Start a new parsing thread.
   		synchronized (parsingThreads) {
   			final Runnable runnable = new Runnable() {
				public void run() {
					parsingThreads.add(this);
					runParserAndHeuristic(this, useCases, parser, heuristic);
				}
		    };
	   		final Thread thread = new Thread(runnable);
   			thread.start();
   		}
	}
	

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#addToAnalyzeModel(ucot.input.UseCase)
	 */
	public void addToAnalyzeModel(Vector<UseCase> useCases) {
		addToAnalyzeModel(useCases, currentParser, currentHeuristic);
	}

	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getAnalyzeModel()
	 */
	public AnalyzeModel getAnalyzeModel() {		
		return analyzeModel;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getParsers()
	 */
	public Vector<ParserInterface> getParsers() {		
		return parsers;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getHeuristics()
	 */
	public Vector<HeuristicInterface> getHeuristics() {
		return heuristics;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getOutputs()
	 */
	public Vector<OutputInterface> getOutputs() { 
		return outputs;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getInputs()
	 */
	public InputCollection getInputs() {
		return inputs;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#setCurrentParser(ucot.parser.ParserInterface)
	 */
	public void setCurrentParser(ParserInterface parser) {
		currentParser = parser;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#setCurrentHeuristic(ucot.heuristic.HeuristicInterface)
	 */
	public void setCurrentHeuristic(HeuristicInterface heuristic) {
		currentHeuristic = heuristic;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getCurrentParser()
	 */
	public ParserInterface getCurrentParser() {
		return currentParser;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getCurrentHeuristic()
	 */
	public HeuristicInterface getCurrentHeuristic() {
		return currentHeuristic;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#output(java.net.URL, ucot.output.OutputInterface, ucot.model.AnalyzeModel)
	 */
	public void output(URL url, OutputInterface output, AnalyzeModel model) throws Exception {
		if ( output != null ) {
			output.output(model, url);
		}
		userInterface.exportDone();
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#saveAnalyzeModel(java.net.URL)
	 */
	public void saveAnalyzeModel(URL url) {
		if ( url == null ) return;
		
		try {
			if ( url.getProtocol().equalsIgnoreCase("file") ) {
				File file = new File(url.getFile());
				if ( !file.canWrite() && file.exists()) {
					userInterface.printError(Messages.formatMessage(FILE_NOT_WRITABLE_ERROR,
							url.getFile()));
					return;
				}
				
				FileOutputStream fileOutputStream = null;
				ObjectOutputStream objectOutputStream = null;
				
				try {
					fileOutputStream = new FileOutputStream(file);
					objectOutputStream = new ObjectOutputStream(fileOutputStream);
					// Save model.
					objectOutputStream.writeObject((ObjectAnalyzeModel)analyzeModel);
				} finally {
					if ( objectOutputStream != null ) objectOutputStream.close();
					if ( fileOutputStream != null ) fileOutputStream.close();
				}
			}
		} catch (Exception e) {
			userInterface.printError(FILE_NOT_SAVED_ERROR);
			logger.warning(StringTools.exceptionStackTrace(e));
		}
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#reloadUseCases(java.net.URL)
	 */
	public void reloadUseCases(URL url) {
		useCases.removeFromUrl(url);
		loadUseCases(url);
	}
	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#applyProperties()
	 */
	public void applyProperties() throws BadPropertyValueException {
		Vector<BadPropertyValueException> exceptions = new Vector<BadPropertyValueException>();
		// Find and set the current parser.
		currentParser = findParser(properties.
				getProperty(PROPERTY_CURRENT_PARSER));
		
		// If the specified one wasn't found.
		if ( currentParser == null && parsers.size() > 0 ) {
			// Select the first one from the list...
			currentParser = parsers.get(0);
			// and update properties.
			properties.setProperty(PROPERTY_CURRENT_PARSER,
					currentParser.getName());
			exceptions.add(new BadPropertyValueException(Messages.formatMessage(
					Messages.getString("Core.PARSER_NOT_FOUND"),
					properties.getProperty(PROPERTY_CURRENT_PARSER))));
		}
		
		// Find and set current heuristic.
		currentHeuristic = findHeuristic(properties.
				getProperty(PROPERTY_CURRENT_HEURISTIC));
		
		// If the specified one wasn't found. 
		if ( currentHeuristic == null && heuristics.size() > 0 ) {
			// Select the first one from the list...
			currentHeuristic = heuristics.get(0);
			// and update properties.
			properties.setProperty(PROPERTY_CURRENT_HEURISTIC,
					currentHeuristic.getName());			
			exceptions.add(new BadPropertyValueException(Messages.formatMessage(
					Messages.getString("Core.HEURISTIC_NOT_FOUND"),
					properties.getProperty(PROPERTY_CURRENT_HEURISTIC))));
		}
		
		// Make input adapters apply their properties.
		for ( int i = 0; i < inputs.getInputCount(); i++ ) {
			try {
				inputs.getInput(i).applyProperties();
			} catch (BadPropertyValueException e) {
				exceptions.add(e);
			}
		}
		
		// Make heuristic adapters to apply their properties.
		for ( HeuristicInterface heuristic : heuristics ) {
			try {
				heuristic.applyProperties();
			} catch (BadPropertyValueException e) {
				exceptions.add(e);
			}
		}
		
		// Make parser adapters to apply their properties.
		for ( ParserInterface parser : parsers ) {
			try {
				parser.applyProperties();
			} catch (BadPropertyValueException e) {
				exceptions.add(e);
			}
		}
		
		// Make output adapters to apply their properties.
		for ( OutputInterface output : outputs ) {
			try {
				output.applyProperties();
			} catch (BadPropertyValueException e) {
				exceptions.add(e);
			}
		}
		
		// Now apply user interface's properties.
		try {
			userInterface.applyProperties();
		} catch (BadPropertyValueException e) {
			exceptions.add(e);
		}
		
		String exceptionMessage = "";
		for ( BadPropertyValueException e : exceptions ) {
			exceptionMessage += "\n" + e.getMessage();
		}
		if ( !exceptions.isEmpty() ) {
			exceptionMessage = exceptionMessage.substring(1);
			throw new BadPropertyValueException(exceptionMessage);
		}
	}
	
	/**
	 * Helper method for applyProperties to find parsers by name.
	 * @param name Parser to search.
	 * @return Found parser or null if no parser with given name existed.
	 */
	private ParserInterface findParser(String name) {		
		for ( int i = 0; i < parsers.size() ; i++ ) {
			if ( parsers.get(i).getName().equals(name) ) {
				return parsers.get(i);
			}
		}		
		return null;
	}
	
	/**
	 * Helper method for applyProperties to find heuristics by name.
	 * @param name Heuristic to search.
	 * @return Found heuristic or null if no heuristic with given name existed.
	 */
	private HeuristicInterface findHeuristic(String name) {
		for ( int i = 0; i < heuristics.size() ; i++ ) {
			if ( heuristics.get(i).getName().equals(name) ) {
				return heuristics.get(i);
			}
		}		
		return null;
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		Properties result = super.loadDefaultProperties();
		// TODO Set default properties here, for example:
		result.setProperty(PROPERTY_CURRENT_PARSER, "Stanford parser");
		result.setProperty(PROPERTY_CURRENT_HEURISTIC, "Abbott's heuristic");
		return result;
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#saveProperties()
	 */
	public synchronized void saveProperties() throws IOException {
		Vector<IOException> exceptions = new Vector<IOException>();
		try {
			super.saveProperties();
		} catch (IOException e) {
			exceptions.add(e);
		}
		
		// Make input adapters save their properties.
		for ( int i = 0; i < inputs.getInputCount(); i++ ) {
			try {
				inputs.getInput(i).saveProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make heuristic adapters to save their properties.
		for ( HeuristicInterface heuristic : heuristics ) {
			try {
				heuristic.saveProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make parser adapters to save their properties.
		for ( ParserInterface parser : parsers ) {
			try {
				parser.saveProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make output adapters to save their properties.
		for ( OutputInterface output : outputs ) {
			try {
				output.saveProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Now save user interface's properties.
		try {
			userInterface.saveProperties();
		} catch (IOException e) {
			exceptions.add(e);
		}
		
		String exceptionMessage = "";
		for ( IOException e : exceptions ) {
			exceptionMessage += "\n" + e.getMessage();
		}
		if ( !exceptions.isEmpty() ) {
			exceptionMessage = exceptionMessage.substring(1);
			throw new IOException(exceptionMessage);
		}
	}

	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadProperties()
	 */
	public synchronized void loadProperties() throws IOException {
		Vector<IOException> exceptions = new Vector<IOException>();
		try {
			super.loadProperties();
		} catch (IOException e) {
			exceptions.add(e);
		}
		
		// Make input adapters load their properties.
		for ( int i = 0; i < inputs.getInputCount(); i++ ) {
			try {
				inputs.getInput(i).loadProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make heuristic adapters to load their properties.
		for ( HeuristicInterface heuristic : heuristics ) {
			try {
				heuristic.loadProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make parser adapters to load their properties.
		for ( ParserInterface parser : parsers ) {
			try {
				parser.loadProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Make output adapters to load their properties.
		for ( OutputInterface output : outputs ) {
			try {
				output.loadProperties();
			} catch (IOException e) {
				exceptions.add(e);
			}
		}
		
		// Now load user interface's properties.
		try {
			userInterface.loadProperties();
		} catch (IOException e) {
			exceptions.add(e);
		}
		
		String exceptionMessage = "";
		for ( IOException e : exceptions ) {
			exceptionMessage += "\n" + e.getMessage();
		}
		if ( !exceptions.isEmpty() ) {
			exceptionMessage = exceptionMessage.substring(1);
			throw new IOException(exceptionMessage);
		}
	}
}
