package ucot.input;

import java.net.URL;
import java.util.List;
import java.util.Observable;
import java.util.Vector;

import ucot.exceptions.CannotLoadUseCasesException;

/**
 * Collection of usecases. Handles finds and merges for them.
 * @author vevijopi
 *
 */
public class UseCaseCollection extends Observable{
	
	protected static String CANNOT_LOAD_USE_CASES_ERROR =
		"Cannot load use cases from the following file(s):";
	
	private Vector<UseCase> useCases;
	
	
	public UseCaseCollection() {
		useCases = new Vector<UseCase>();
	}
	
	/**
	 * Merges given UseCaseCollection to this 
	 * @param collection UseCaseCollection to merge
	 */
	public synchronized void merge(UseCaseCollection collection) throws CannotLoadUseCasesException {
	boolean changed = false;
		if ( collection != null ) {

			synchronized(collection){
				//Go through all usecases from the another collection
				for(int i=0;i<collection.getUseCaseCount();i++) {					
					//Add each usecase to collection (addUseCase checks if they already exist in the collection)
					if (addUseCase(collection.getUseCase(i),false))
						changed = true; //addUseCase returns true if anything was added
				}
			}
		} else {
			String message = CANNOT_LOAD_USE_CASES_ERROR;
			for ( URL file : collection.getURLs() ) {
				message += "\n  " + file.getFile();
			}
			throw new CannotLoadUseCasesException(message);
		}
		
		if (changed) {
			setChanged();
        	notifyObservers();
		}

	}
	
	/**
	 * Checks if given UseCase exists in this collection
	 * @param usecase UseCase to test
	 * @return true if UseCase exists in this collection
	 */
	public boolean exists(UseCase usecase) {
		if (usecase == null)
			return false;
		return useCases.contains(usecase);
	}
	
	/**
	 * Returns the count of use cases inside this collection.
	 * @return The count of use cases.
	 */
	public int getUseCaseCount() {
		return useCases.size();
	}

    /**
     * Add usecase to collection and notify observers
     * @param usecase UseCase to add
     * @return true if usecase was added
     */
	public boolean addUseCase(UseCase usecase) {
		return addUseCase(usecase,true);
	}
	
	/**
     * Add usecase to collection and notify observers
     * @param usecase UseCase to add
     * @param notify should we notifyobservers and set usecase as changed
     * @return true if usecase was added
     */
	public boolean addUseCase(UseCase usecase, boolean notify) {
		if (usecase == null)
			return false;
		if (exists(usecase))
			return false;
		if (usecase.getStepCount() < 1)
			return false;
		
		useCases.add(usecase);
		
		if (notify) {
			setChanged();
        	notifyObservers();
		}
        
        return true;
	}
	
	
	
	
	/**
	 * Finds usecase that has the given id
	 * @param id id to find
	 * @return usecase with given id, -1 if none found
	 */
	public UseCase find(String id) {
		if (id == null) 
			return null;
		
		for(int i=0;i<getUseCaseCount();i++) {			
			if (getUseCase(i).getUseCaseId().equals(id))
				return getUseCase(i);			
		}
		
		return null;
	}
	
    /**
     * Returns UseCase from index
     * @param index
     * @return UseCase
     */
	public UseCase getUseCase(int index) {
		return useCases.get(index);
	}
	
	/**
	 * Removes all usecases that are loaded from given url
	 * @param url usecases that are loaded from this url are removed
	 */
	public void removeFromUrl(URL url) {
		boolean changed = false;
		for(int i=0;i<getUseCaseCount();i++) {
			if (getUseCase(i).getUrl().equals(url)) {
				remove(i);
				i--;
				changed = true;
			}
		}
		
		if (changed) {
			setChanged();
    		notifyObservers();
		}
	}
	   
    /**
     * Returns list of source URLs
     * @return list of source URLs
     */
    public List<URL> getURLs() {
        
        List<URL> urls = new Vector<URL>();
        
        for (UseCase u: useCases) {

            // if url is not on the list, add it
            if (! urls.contains(u.getUrl()) )  {
                urls.add(u.getUrl());
            }
        }
        return urls;
    }
    
    /**
     * Returns list of usecases from source described in url.
     * @param url
     * @return list of usecases from source described in url
     */
    public List<UseCase> getUseCasesFromURL(URL url) {
    	if (url == null)
    		return null;
    	
        List<UseCase> cases = new Vector<UseCase>();
        
        for (UseCase u: useCases) {
            
            if ( u.getUrl().equals(url) ) {
                cases.add(u);
            }
        }
        
        return cases;
    }
    
    /**
     * Clears loaded usecases.
     *
     */
	public void clear(){
		useCases.clear();
        setChanged();
        notifyObservers();
	}
    
	/**
	 * Removes usecase with given index
	 * @param index index of the usecase to remove
	 */
	public void remove(int index) {
		useCases.remove(index);
	}
	
	/**
	 * Removes all usecases that have been marked as subusecase
	 *
	 */
	public void removeAllSubUseCase() {
		for(int i=0;i<useCases.size();i++) {
			if (getUseCase(i).isSubUseCase()) {
				remove(i);
				i--;
			}
		}
	}
	
	/**
	 * Marks all usecases within this collection as unanalyzed
	 *
	 */
	public void markAllUnanalyzed() {
		for ( int i=0; i < useCases.size(); i++ ) {
			markAllUnanalyzed(useCases.get(i));
        }
		setChanged();
        notifyObservers();
	}
	
	/**
	 * Marks given usecase and it's sub usecases as unanalyzed
	 * @param usecase usecase to be marked
	 */
    private void markAllUnanalyzed(UseCase usecase) {
        if (usecase == null) return;
        for (int i = 0; i < usecase.getStepCount(); i++) {
            markAllUnanalyzed(usecase.getStep(i).getSubUseCase());
        }
        usecase.setUseCaseAnalyzed(false);
        
    }
    
    
    /**
     * Stores all usecases within this collection to a string and returns it
     */
    public String toString() {
    	StringBuffer sb = new StringBuffer();
    	sb.append(useCases.size() + " UseCases :\n");
    	
    	for(int i=0;i<useCases.size();i++) {
    		sb.append(useCases.get(i).toString() + "\n");
    	}
    	
    	return sb.toString();
    }
    
    /**
	 * Resolves step and sub-usecase relations, removes sub-usecases from usecasecollection
	 * and stores their reference to the corresponding usecase step
	 * @param markAsSubUseCases 
	 */
	public void resolveSubUseCases(boolean markAsSubUseCases) {		
		//Go through all usecases in usecasecollection
		for(int i=0;i<getUseCaseCount();i++) {			
			resolveSubUseCases(getUseCase(i),markAsSubUseCases);			
		}
		
		//Removes all subusecase from the main list
		removeAllSubUseCase();
	}
	
	/**
	 * Resolves given usecases step's sub-usecases and marks
	 * them as sub-usecases if necessary 
	 * @param markAsSubUseCases 
	 */
	public void resolveSubUseCases(UseCase usecase,boolean markAsSubUseCases) {		
		UseCase subUseCase = null;	
		UseCaseStep step = null;				
		
		//go through all steps in each usecase
		for(int j=0;j<usecase.getStepCount();j++) {
			step = usecase.getStep(j);			
			
			//Find steps sub-usecase using steps subusecaseid
			subUseCase = find(step.getSubUseCaseId());				
			
			//If a matching subusecase was found
			if (subUseCase != null) {
				//Set it as the step's sub usecase
				step.setSubUseCase(subUseCase);
				
				//Should the sub-usecase be marked as sub-usecase
				if (markAsSubUseCases)
					subUseCase.setAsSubUseCase(true);	
				
				//If the step has no name, then we get it's name from sub usecases name
				if (step.getStep().length() < 1) 
					step.setStep(subUseCase.getName());	
			}
		}
	}
} 
