package ucot.model.object;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import ucot.utils.StringTools;
import edu.stanford.nlp.util.StringUtils;
/**
 * This class represents entity in analyze model. 
 * @author pajumasu
 */
public class Entity implements Serializable {
	public static final long serialVersionUID = 2368572046870269076L;
	
	/**
	 * Maps attribute names and attributes.
	 * Attribute names are not in use currently.
	 */
	private Map<String, Attribute> attributes = new HashMap<String, Attribute>();
	
	/**
	 * Maps method names and methods.
	 */
	private Map<String,Method> methods = new HashMap<String,Method>();
	
	/**
	 * The child entities.
	 */
	private Set<Entity> childEntities = new HashSet<Entity>();
	
	/**
	 * The parent entities.
	 */
	private Set<Entity> parentEntities = new HashSet<Entity>();
	
	/**
	 * The name of this entity.
	 */
	private String name;
	
	
	/**
	 * The type of the entity.
	 */
	private String type;
	
	/**
	 * Marks if this entity is deleted or not.
	 * Because there are many references to entities inside the model
	 * it is easier to mark entity deletet when
	 * its is removed from the model and the unnessesary references are removed
	 * when they are needed. This way we dont need to know
	 * all the places for the references and remove them on site
	 * when entity is deleted.
	 */
	boolean deleted = false;
	
	/**
	 * Creates the entity.
	 * The name is modified so that the first letter is always
	 * an uppercase letter.
	 * @param name
	 */
	public Entity(String name){
		setName(name);
	}

	/**
	 * Sets the name of the entity.
	 * The name is modified so that the first letter is always
	 * an uppercase letter.
	 * @param name
	 */
	protected void setName(String name){
		this.name = StringTools.firstLetterUpcase(name);		 
	}
	
	/**
	 * Removes deleted entities from the 
	 * attribute list. 
	 */
	private void updateAttributes(){
		for (Iterator<Map.Entry<String, Attribute>> it = attributes.entrySet().iterator(); it.hasNext();){
			Map.Entry<String,Attribute> entry = it.next();
			Entity entity = entry.getValue().getEntity();
			if (entity.deleted) it.remove();
		}
	}
	
	
	/**
	 * Adds attribute for this entity
	 * @param name
	 * @param attribute
	 */
	public void addAttribute(String name, Attribute attribute){
		attributes.put(StringUtils.capitalize(name),  attribute);
	}

	/**
	 * Gets attribute called name.
	 * @param name
	 */
	public Attribute getAttribute(String name){
		updateAttributes();
		return attributes.get(StringUtils.capitalize(name));
	}
	
	/**
	 * Get all the attributes.
	 * @return set of arguments
	 */
	public Set<Attribute> getAttributes(){
		updateAttributes();
		Set<Attribute> ret= new HashSet<Attribute>();
		ret.addAll(attributes.values());
		return ret;
	}
	
	/**
	 * Removes attribute called name.
	 * @param name
	 */
	public void removeAttribute(String name){
		attributes.remove(name);
	}
		
	/**
	 * Removes given attribute. 
	 * @param toBeRemoved
	 */
	public void removeAttribute(Attribute toBeRemoved){
		for (Iterator<Map.Entry<String,Attribute>> it = attributes.entrySet().iterator(); it.hasNext();){
			Map.Entry<String,Attribute> entry = it.next();
			Attribute attribute = entry.getValue();
			if (attribute.equals(toBeRemoved)){
				it.remove();
			}
		}
	}

	
	/**
	 * Adds attribute.
	 * Attributes name is set basend on the
	 * name of the entity defined in attribute.
	 * @param attribute attribute to add
	 */
	public void addAttribute(Attribute attribute){
		attributes.put(attribute.getEntity().getName(), attribute);
	}

	/**
	 * Returns the name of this entity.
	 * @return name of this entity.
	 */
	public String getName(){
		return name;
	}
	
	private void checkEntityCollection(Collection<Entity> col){
		for (Iterator<Entity> it = col.iterator(); it.hasNext();){
			Entity entity = it.next();
			if (entity.deleted){
				it.remove();
				entity.removeChild(this);
			}
		}
	}
	
	/**
	 * Returns the parent entities of this entity.
	 * @return parent entities of this entity
	 */
	public Set<Entity> getParents(){
		checkEntityCollection(parentEntities);		
		Set<Entity> ret = new HashSet<Entity>();
		ret.addAll(parentEntities);
		return parentEntities;
	}
	
	/**
	 * Returns the children of this entity.
	 * @return set of children
	 */
	public Set<Entity> getChildren(){
		checkEntityCollection(childEntities);
		Set<Entity> ret = new HashSet<Entity>();
		ret.addAll(childEntities);
		return ret;
	}

	/**
	 * Adds parent and adds this entity as a child of the 
	 * given entity if nessesary
	 * @param entity
	 */
	public void addParent(Entity entity){
		if (null == entity) return;
		if (this == entity) return;
		if (entity.deleted) return;
		if (parentEntities.add(entity))
			entity.addChild(this);
	}

	/**
	 * Adds child and adds this entity as a parent of the 
	 * given entity if nessesary
	 * @param entity
	 */	
	public void addChild(Entity entity){
		if (null == entity) return;
		if (this == entity) return;
		if (entity.deleted) return;
		if (childEntities.add(entity))
			entity.addParent(this);
	}

	/**
	 * Removes parent and removes this entity from the child list
	 * of a given entity if nessesary.
	 * @param entity
	 */	
	public void removeParent(Entity entity){
		if (null == entity) return;
		if (parentEntities.remove(entity))
			entity.removeChild(this);
	}
	
	/**
	 * Removes child and removes this entity from the parent list
	 * of a given entity if nessesary.
	 * @param entity
	 */	
	public void removeChild(Entity entity){
		if (null == entity) return;
		if (childEntities.remove(entity))
			entity.removeParent(this);
	}
	
	/**
	 * Return method.
	 * @param methodName
	 * @return method
	 */
	public Method getMethod(String methodName){
		return methods.get(methodName.toLowerCase());		
	}
	
	/**
	 * Creates and adds method using given name.
	 * @param methodName name to use
	 */	
	public void addMethod(String methodName){
		Method method = methods.get(methodName.toLowerCase());
		if (null == method){
			method = new Method(methodName);
			methods.put(method.getName(), method);
		}
		
	}
	
	/**
	 * Adds method.
	 * @param method to add
	 */	
	public void addMethod(Method method){
			methods.put(method.getName(), method);
	}
	
	/**
	 * Removes method.
	 * @param methodName
	 */
	public void removeMethod(String methodName){
		methods.remove(methodName);
	}
	
	/**
	 * Returns the names of the methods. 
	 * @return names of the methods
	 */
	public Set<String> getMethodNames(){
		return methods.keySet();
	}

	/**
	 * Return the type of the entity.
	 * @return The type.
	 */
	public String getType() {
		return type;
	}

	/**
	 * Sets the tyoe of the entity.
	 * @param type The type
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Get all the methods.
	 * @return set of methods.
	 */
	public Set<Method> getMethods(){
		Set<Method> ret= new HashSet<Method>();
		ret.addAll(methods.values());
		return ret;
	}
}
