package ucot.model.object;


import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.NoSuchEntityException;
import ucot.model.Updation;
import ucot.model.Updation.UpdationDataType;
import ucot.model.Updation.UpdationType;

/**
 * Editor for the ObjectAnalyzeModel.
 * @author pajumasu
 *
 */
public class ObjectAnalyzeModelEditor implements ModelEditor, Serializable {
	public static final long serialVersionUID = 892359043723230577L;
	
	ObjectAnalyzeModel model;
	
	
	List<Updation> updations = new LinkedList<Updation>();
	
	
	public ObjectAnalyzeModelEditor(ObjectAnalyzeModel model){
		this.model = model;
	}
	
	
	private void sendUpdation(Updation updation){
		model.setChanged();
		model.notifyObservers(updation);
		synchronized(updation){
			updations.add(updation);
		}
	}
	
	public void clearModel() {
		model.clear();
		sendUpdation(new Updation(UpdationType.CLEAR));
	}
	
	public Set<String> getEntityNames() {		
		return model.getEntityNames();
	}
	
	
	
	
	public String getEntityType(String name) {
		Entity entity = model.getEntity(name);
		if (null == entity) return "";

		return entity.getType();
	}


	public void setEntityType(String name, String entityType) {
		if (null == entityType) return;
		Entity entity = model.getEntity(name);
		if (null == entity) return;
		sendUpdation(new Updation(UpdationType.MODIFY, UpdationDataType.ENTITY_TYPE, name, entityType));
		entity.setType(entityType);		
	}


	public void addEntity(String name) {
		Entity entity = model.getEntity(name);
		if (null != entity) return;
		entity = new Entity(name);
		model.addEntity(entity);
		sendUpdation(new Updation(UpdationType.NEW, UpdationDataType.ENTITY, name));
	}
	
	public void removeEntity(String name) {		
		model.removeEntity(name);
		sendUpdation(new Updation(UpdationType.DELETION, UpdationDataType.ENTITY, name));
	}
	
	public void changeEntityName(String oldName, String newName) {
		Entity entity = model.getEntity(oldName);
		if (null != entity){
			model.removeEntity(oldName);
			entity.setName(newName);
			model.addEntity(entity);
			sendUpdation(new Updation(UpdationType.MODIFY, UpdationDataType.ENTITY, oldName, newName));
		} else {
			throw new Error("No such entity");
		}
	}
	
	public boolean containsEntity(String entityName) {
		return model.getEntityNames().contains(entityName);
	}
	
	public void addParent(String entityName, String parentEntity) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){
			Entity parent = model.getEntity(parentEntity);
			if (null == parent) throw new NoSuchEntityException(parentEntity);
			entity.addParent(parent);
			sendUpdation(new Updation(UpdationType.NEW, UpdationDataType.ENTITY_PARENT, entityName,parentEntity));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void removeParent(String entityName, String parentEntity) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){
			Entity parent = model.getEntity(parentEntity);
			if (null == parent) throw new Error("No such entity " + parentEntity);
			entity.removeParent(parent);
			sendUpdation(new Updation(UpdationType.DELETION, UpdationDataType.ENTITY_PARENT, entityName, parentEntity));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void removeAllParents(String entityName) {
		model.updationStartedSignal();
		try{
			Entity entity = model.getEntity(entityName);
			if (null != entity){
				Set<Entity> parents = entity.getParents();
				for(Entity parent : parents){
					entity.removeParent(parent);
				}
			} else {
				throw new Error("No such entity " + entityName );
			}
		}finally{
			model.readySignal();
		}
	}
	
	public Set<String> getParents(String entityName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){
			Set<Entity> parents = entity.getParents();
			Set<String> ret = new HashSet<String>();
			for(Entity parent : parents){
				ret.add(parent.getName());
			}
			return ret;
		} else {
			return new HashSet<String>();
		}
	}
	
	public void addChild(String entityName, String childEntity) {
		addParent(childEntity, entityName);
	}
	
	public void removeChild(String entityName, String childEntity) {
		removeParent(childEntity, entityName);
	}
	
	public void removeAllChildren(String entityName) {
		model.updationStartedSignal();
		try{
			Entity entity = model.getEntity(entityName);
			if (null != entity){
				Set<Entity> childs = entity.getChildren();
				for(Entity child : childs){
					entity.removeChild(child);
				}
			} else {
				throw new Error("No such entity " + entityName );
			}
		}finally{
			model.readySignal();
		}
	}
	
	public Set<String> getChildren(String entityName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){
			Set<Entity> children = entity.getChildren();
			Set<String> ret = new HashSet<String>();
			for(Entity child : children){
				ret.add(child.getName());
			}
			return ret;
		} else {
			return new HashSet<String>();
			
		}
	}
	
	public void addMethod(String entityName, String methodName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			entity.addMethod(methodName);
			sendUpdation(new Updation(UpdationType.NEW, UpdationDataType.METHOD, entityName, methodName));
		} else {
			throw new Error("No such entity " + entityName );
		}
		
	}
	
	public void removeMethod(String entityName, String methodName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			entity.removeMethod(methodName);
			sendUpdation(new Updation(UpdationType.DELETION, UpdationDataType.METHOD, entityName, methodName));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void changeMethodName(String entityName, String methodOldName,
			String methodNewName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Method method = entity.getMethod(methodOldName);
			if (null == method) throw new Error("No such entity " + methodOldName + " in " + entityName );
			entity.removeMethod(methodOldName);
			method.setName(methodNewName);
			entity.addMethod(method);
			sendUpdation(new Updation(UpdationType.MODIFY, UpdationDataType.METHOD, entityName, methodOldName, methodNewName));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public boolean containsMethod(String entityName, String methodName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			return entity.getMethodNames().contains(methodName);
		} else {
			return false;
		}
	}
	
	public Set<String> getMethodNames(String entityName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			return entity.getMethodNames();
		} else {
			return new HashSet<String>();
		}
	}
	
	public Set<String> getEntitiesInfluencedByMethod(String entityName,
			String methodName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Method method = entity.getMethod(methodName);
			Set<String> ret = new HashSet<String>();
			if (null == method) return ret;
			Set<Entity> influences = method.getInfluenced();
			
			for(Entity influence : influences){
				ret.add(influence.getName());
			}
			return ret;
		} else {
			return new HashSet<String>();
		}
		
	}
	
	public void addEntityInfluenceByMethod(String entityName,
			String methodName, String influencedEntity) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			addMethod(entityName,methodName);
			Method method = entity.getMethod(methodName);
			if (null == method) throw new Error("No such entity " + methodName + " in " + entityName );
			Entity infEntity = model.getEntity(influencedEntity);
			if (null == infEntity) throw new Error("No such entity " + influencedEntity);
			method.addInfluence(infEntity);
			sendUpdation(new Updation(UpdationType.NEW, UpdationDataType.METHOD_INFLUENCE, entityName, methodName, influencedEntity));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void removeEntityInfluenceByMethod(String entityName,
			String methodName, String influencedEntity) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Method method = entity.getMethod(methodName);
			if (null == method) throw new Error("No such entity " + methodName + " in " + entityName );
			Entity infEntity = model.getEntity(influencedEntity);
			if (null == infEntity) throw new Error("No such entity " + influencedEntity);
			method.removeInfluence(infEntity);
			sendUpdation( new Updation(UpdationType.DELETION, UpdationDataType.METHOD_INFLUENCE, entityName, methodName, influencedEntity) );
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void addAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Entity attributeEntity = model.getEntity(attributeName);
			if (null == attributeEntity) throw new Error("No such entity " + attributeName);
			Attribute attribute = new Attribute(attributeEntity);
			entity.addAttribute(attribute);
			sendUpdation(new Updation(UpdationType.NEW, UpdationDataType.ATTRIBUTE, entityName, attributeName));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void removeAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Attribute attribute = entity.getAttribute(attributeName);
			if (null == attribute) throw new Error("No such entity " + attribute);
			entity.removeAttribute(attribute);
			sendUpdation(new Updation(UpdationType.DELETION, UpdationDataType.ATTRIBUTE, entityName, attributeName));
		} else {
			throw new Error("No such entity " + entityName );
		}
		
	}
	
	public boolean containsAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			return entity.getAttribute(attributeName) != null;
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public String getAttributeFromCardinal(String entityName,
			String attributeName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Attribute attribute = entity.getAttribute(attributeName);
			if (null == attribute) throw new Error("No such entity " + attribute);
			return attribute.fromCardinal;
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public String getAttributeToCardinal(String entityName, String attributeName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Attribute attribute = entity.getAttribute(attributeName);
			if (null == attribute) throw new Error("No such entity " + attributeName);
			return attribute.toCardinal;
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void setAttributeFromCardinal(String entityName,
			String attributeName, String cardinal) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Attribute attribute = entity.getAttribute(attributeName);
			if (null == attribute) throw new Error("No such entity " + attribute);
			attribute.fromCardinal = cardinal;
			sendUpdation(new Updation(UpdationType.MODIFY, UpdationDataType.ATTRIBUTE_FROM_CARDINALITY, entityName, attributeName, cardinal));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public void setAttributeToCardinal(String entityName, String attributeName,
			String cardinal) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Attribute attribute = entity.getAttribute(attributeName);
			if (null == attribute) throw new Error("No such entity " + attribute);
			attribute.toCardinal = cardinal;
			sendUpdation(new Updation(UpdationType.MODIFY, UpdationDataType.ATTRIBUTE_TO_CARDINALITY, entityName, attributeName, cardinal));
		} else {
			throw new Error("No such entity " + entityName );
		}
	}
	
	public Set<String> getAttributeNames(String entityName) {
		Entity entity = model.getEntity(entityName);
		if (null != entity){ 
			Set<Attribute> attributes = entity.getAttributes();
			Set<String> ret= new HashSet<String>();
			for(Attribute attribute : attributes){
				ret.add(attribute.getEntity().getName());
			}
			return ret;
			
		} else {
			return new HashSet<String>();
		}
	}
	
	public void merge(AnalyzeModel fromModel) {
		if (fromModel == null) return;
		model.updationStartedSignal();
		try{
			ModelEditor fromEditor = fromModel.getEditor();
			Set<String> fromEntities = fromEditor.getEntityNames();
			Set<String> currentEntities = this.getEntityNames();
			
			Set<String> newEntities = new HashSet<String>();
			newEntities.addAll(fromEntities);
			newEntities.removeAll(currentEntities);
			
			Set<String> missingEntities = new HashSet<String>();
			missingEntities.addAll(currentEntities);
			missingEntities.removeAll(fromEntities);
			
			
			Set<String> possiblyChangedEntities = new HashSet<String>();
			possiblyChangedEntities.addAll(fromEntities);
			possiblyChangedEntities.removeAll(missingEntities);
			
			// Create new entities.
			for(String name : newEntities){
				addEntity(name);
			}
			
			// Update possibly changed entities.
			for(String entityName : possiblyChangedEntities){
				
				// Methods
				Set<String> fromMethods = fromEditor.getMethodNames(entityName);
				Set<String> currentMethods = this.getMethodNames(entityName);
				
				Set<String> newMethods = new HashSet<String>();
				newMethods.addAll(fromMethods);
				newMethods.removeAll(currentMethods);
				
				Set<String> missingMethods = new HashSet<String>();
				newMethods.addAll(currentMethods);
				newMethods.removeAll(fromMethods);
				
				Set<String> possiblyChangedMethods = new HashSet<String>();
				possiblyChangedMethods.addAll(fromMethods);
				possiblyChangedMethods.removeAll(missingMethods);
				
				// Add new methods.
				for(String method : newMethods){
					addMethod(entityName, method);
				}
				
				// Check method influences
				for(String method : possiblyChangedMethods){
					Set<String> fromInfluenced = fromEditor.getEntitiesInfluencedByMethod(entityName, method);
					Set<String> currentInfluenced = this.getEntitiesInfluencedByMethod(entityName,method);
					
					Set<String> newInfluences = new HashSet<String>();
					newInfluences.addAll(fromInfluenced);
					newInfluences.removeAll(currentInfluenced);
					
					for(String influence : newInfluences){
						addEntityInfluenceByMethod(entityName,method,influence);					
					}				
				}
				
				
				// Attributes
				Set<String> fromAttributes = fromEditor.getAttributeNames(entityName);
				Set<String> currentAttributes = this.getAttributeNames(entityName);
				
				Set<String> newAttributes  = new HashSet<String>();
				newAttributes.addAll(fromAttributes);
				newAttributes.removeAll(currentAttributes);
				
				for(String attributeName : newAttributes){
					String fromCardinal = fromEditor.getAttributeFromCardinal(entityName,attributeName);
					String toCardinal = fromEditor.getAttributeToCardinal(entityName,attributeName);
					
					this.addAttribute(entityName,attributeName);
					this.setAttributeFromCardinal(entityName, attributeName, fromCardinal);
					this.setAttributeToCardinal(entityName, attributeName, toCardinal);
					
					
				}
				
				
				// Parents
				Set<String> fromParents = fromEditor.getParents(entityName);
				Set<String> currentParents = this.getParents(entityName);
				
				Set<String> newParents  = new HashSet<String>();
				newParents.addAll(fromParents);
				newParents.removeAll(currentParents);
				
				for(String parentName : newParents){
					this.addParent(entityName, parentName);								
				}	
			}
		} finally{
			model.readySignal();
		}
	}

    public List<Updation> getUpdations() {
		return Collections.unmodifiableList(updations);
	}
	
	
	public void stepBack(int steps) {
		
	}
	
	public boolean execute(Updation updation) {
		switch(updation.getType()){
		case NEW:			
			switch(updation.getDataType()){
			case ENTITY:
				String[] ref = updation.getReferences();
				String entity = ref[0];				
				addEntity(entity);
				return true;											
			case ATTRIBUTE:
				ref = updation.getReferences();
				entity = ref[0];				
				String attribute = ref[1];
				addAttribute(entity,attribute);
				return true;											
			case METHOD:
				ref = updation.getReferences();
				entity = ref[0];
				String method = ref[1];				
				addMethod(entity,method);
				return true;								
			case METHOD_INFLUENCE:
				ref = updation.getReferences();
				entity = ref[0];
				method = ref[1];
				String infEntity = ref[2];
				addEntityInfluenceByMethod(entity,method,infEntity);
				return true;								
			case ENTITY_PARENT:
				ref = updation.getReferences();
				entity = ref[0];
				String parent = ref[1];
				addParent(entity,parent);
				return true;				
			default:
				// Unknown creation
				return false;
			}
		case DELETION:
			switch(updation.getDataType()){
			case ENTITY:
				String[] ref = updation.getReferences();
				String entity = ref[0];				
				removeEntity(entity);
				return true;											
			case ATTRIBUTE:
				ref = updation.getReferences();
				entity = ref[0];				
				String attribute = ref[1];
				removeAttribute(entity,attribute);
				return true;											
			case METHOD:
				ref = updation.getReferences();
				entity = ref[0];
				String method = ref[1];				
				removeMethod(entity,method);
				return true;								
			case METHOD_INFLUENCE:
				ref = updation.getReferences();
				entity = ref[0];
				method = ref[1];
				String infEntity = ref[2];
				removeEntityInfluenceByMethod(entity,method,infEntity);
				return true;								
			case ENTITY_PARENT:
				ref = updation.getReferences();
				entity = ref[0];
				String parent = ref[1];
				removeParent(entity,parent);
				return true;				
			default:
				// Unknown deleltion
				return false;
			}
		case MODIFY:
			switch(updation.getDataType()){
			case ENTITY:
				String[] ref = updation.getReferences();
				String entity = ref[0];				
				String newEntity = ref[1];
				changeEntityName(entity,newEntity);
				return true;											
			case ATTRIBUTE_TO_CARDINALITY:
				ref = updation.getReferences();
				entity = ref[0];				
				String attribute = ref[1];
				String cardinal = ref[2];
				setAttributeToCardinal(entity, attribute, cardinal);
				return true;
			case ATTRIBUTE_FROM_CARDINALITY:
				ref = updation.getReferences();
				entity = ref[0];				
				attribute = ref[1];
				cardinal = ref[2];
				setAttributeFromCardinal(entity, attribute, cardinal);
				return true;											
			case METHOD:
				ref = updation.getReferences();
				entity = ref[0];
				String method = ref[1];				
				String newMethod = ref[2];
				changeMethodName(entity,method, newMethod);
				return true;													
			default:
				// Unknown modify.
				return false;
			}
		case CLEAR:
			clearModel();
			return true;
			
		}
		return false;
	}


	/* (non-Javadoc)
	 * @see ucot.model.ModelEditor#mergeEntity(java.lang.String, java.util.Set)
	 */
	public void mergeEntity(String targetEntityName, Set<String> mergeSet) {
		Entity unity = model.getEntity(targetEntityName);
		if (null == unity) {
			addEntity(targetEntityName);
			unity = model.getEntity(targetEntityName);
			assert unity == null; // We just created it so it should not be null..
		}
		
		// Build set of entities instead of names.
		Set<Entity> mergeEntities  = new HashSet<Entity>();
		for (String entityName : mergeSet){
			Entity entity = model.getEntity(entityName);
			mergeEntities.add(entity);
		}
		
		
		// Change all reference to listed entities to
		// references to unity entity.
		Set<String> entityNames = model.getEntityNames();
		for(String entityName : entityNames){
			Entity entity = model.getEntity(entityName);
			
			Set<Attribute> attributes = entity.getAttributes();
			for(Attribute attribute : attributes){
				if (mergeEntities.contains(attribute.getEntity())){
					attribute.setEntity(unity);		
				}
			}
			
			Set<Method> methods = entity.getMethods();
			for(Method method : methods){
				Set<Entity> influenced = method.getInfluenced();
				for (Entity infEntity : influenced){
					if (mergeEntities.contains(infEntity)){
						method.removeInfluence(infEntity);
						method.addInfluence(unity);
					}
				}
			}
			// Parents
			Set<Entity> parents = entity.getParents();
			for(Entity parent : parents){
				if (mergeEntities.contains(parent)){
					entity.removeParent(parent);
					entity.addParent(unity);
				}
			}
			
			// Children
			Set<Entity> children = entity.getChildren();
			for(Entity child : children){
				if (mergeEntities.contains(child)){
					entity.removeChild(child);
					entity.addChild(unity);
				}
			}

		}
		
		

		// Merge all information from the merge entities
		for(Entity entity : mergeEntities){
			
			// Methods
			Set<Method> methods = entity.getMethods();
			for(Method  method : methods){
				unity.addMethod(method);
			}
			
			// Attributes
			Set<Attribute> attributes = entity.getAttributes();
			for(Attribute attribute : attributes){
				unity.addAttribute(attribute);
			}

			// Parents
			Set<Entity> parents = entity.getParents();
			for(Entity parent : parents){
				unity.addParent(parent);
			}

			// Parents
			Set<Entity> children = entity.getChildren();
			for(Entity child : children){
				unity.addChild(child);
			}

		}
		
		// Remove merged entities.
		for(Entity entity : mergeEntities){
			model.removeEntity(entity);
		}
		
		sendUpdation(new Updation(UpdationType.READY));
		
	}
	
}
