package ucot.ui.gui.dot;

import java.awt.Color;

/**
 * This class implements a color theme for dot markup language,
 * which is useful for DotPanel when figuring out color strings.
 * It is easy to change the color theme using the method
 * 'changeToColorTheme' and giving it the identifier of the
 * preferred color theme.
 * 
 * All themes available currently have to be hard coded here
 * because of the way how dot understands colors.
 * 
 * TODO Make color themes more dynamic?
 * TODO Add more color themes.
 * 
 * @author tujupien
 */
public class DotColorTheme {
    private DotColorModel normal;
    private DotColorModel highlight;
    
    // Initialize default color theme initializer.
    private final static DotColorModel.ColorModel DEFAULT_HIGHLIGHT =
    	DotColorModel.ColorModel.BLUE;
    private final static DotColorModel.ColorModel DEFAULT_COLOR =
    	DotColorModel.ColorModel.BLACK_ON_WHITE;
    
    /**
     * Normal color changer.
     * @param color New normal color.
     */
	public void changeColor(String color) {
		normal = DotColorModel.color(color);
	}
	
	/**
     * Normal color changer.
     * @param color New normal color.
     */
	public void changeColor(DotColorModel.ColorModel color) {
		normal = DotColorModel.color(color);
	}
	
	/**
	 * Highlight color changer.
	 * @param highlight New highlight color.
	 */
	public void changeHighlight(String highlight) {
		this.highlight = DotColorModel.color(highlight);
	}
	
	/**
	 * Highlight color changer.
	 * @param highlight New highlight color.
	 */
	public void changeHighlight(DotColorModel.ColorModel highlight) {
		this.highlight = DotColorModel.color(highlight);
	}
	
	/**
	 * Constructor for DotColorTheme class.
	 * @param color Normal color of the graph.
	 * @param highlight Color of the highlighted elements.
	 */
    public DotColorTheme(DotColorModel.ColorModel color,
    		DotColorModel.ColorModel highlight) {
    	changeColor(color);
    	changeHighlight(highlight);
    }
    
    /**
     * Constructor for DotColorTheme class.
     * @param color Normal color of the graph.
     */
    public DotColorTheme(DotColorModel.ColorModel color) {
    	this(color, DEFAULT_HIGHLIGHT);
    }
    
    /**
     * Default constructor for DotColorTheme class which initially
     * uses the default color theme.
     */
    public DotColorTheme() {
    	this(DEFAULT_COLOR);
    }
    
    /**
     * Method which formats highlighted item's color attributes
     * into dot's syntax.
     * @return Highlight nodes' or edges' string in dot's syntax.
     */
    public String getHighlightedColorString() {
    	return "style=\"filled" +
    		"\", color=\"" + highlight.getEdgeColor() +
    	    "\", fillcolor=\"" + highlight.getFillColor() +
    	    "\", fontcolor=\"" + highlight.getFontColor() + "\"";
    }
    
    /**
     * Method for getting the background color of the whole graph
     * in dot's syntax.
     * @return Background color string in dot's syntax.
     */
    public String getBackgroundColorString() {
    	return "bgcolor=\"" + normal.getFillColor() + "\"";
    }

    /**
     * Method which formats normal item's color attributes
     * into dot's syntax.
     * @return Normal nodes' or edges' string in dot's syntax.
     */
    public String getNormalColorString() {
    	return "style=\"filled" +
		"\", color=\"" + normal.getEdgeColor() +
	    "\", fillcolor=\"" + normal.getFillColor() +
	    "\", fontcolor=\"" + normal.getFontColor() + "\"";
    }
    
    /**
     * Method for getting the appropriate color string for
     * an entity based on its highlight status.
     * @param isHighlighted Defines wether or not the returned
     *   color string is supposed to be for an highlighted entity
     *   or a normal entity.
     * @return Returns appropriate color string in dot's syntax.
     */
    public String getColorString(boolean isHighlighted) {
    	if ( isHighlighted ) return getHighlightedColorString();
    	return getNormalColorString();
    }
    
    /**
     * Method for acquiring the name of the current color model.
     * @return Name of the color model.
     */
    public String getColor() {
    	return this.normal.getModel();
    }
    
    /**
     * Method for acquiring the name of the current highlight model.
     * @return Name of the highlight model.
     */
    public String getHighlight() {
    	return this.highlight.getModel();
    }
    
    /**
     * Method for getting the background color of the graph as
     * a java object. This helps to figure out the background
     * color of the DotPanel.
     * @return Graph background color as a Java object.
     */
    public Color getBackgroundColorAsJavaObject() {
    	// TODO Complete this dot color conversion map.
    	if ( normal.getFillColor().equalsIgnoreCase("white") ) return Color.WHITE;
    	if ( normal.getFillColor().equalsIgnoreCase("black") ) return Color.BLACK;
    	if ( normal.getFillColor().equalsIgnoreCase("red") ) return Color.RED;
    	if ( normal.getFillColor().equalsIgnoreCase("green") ) return Color.GREEN;
    	if ( normal.getFillColor().equalsIgnoreCase("blue") ) return Color.BLUE;
    	
    	// Set default background color here in case the following
    	// dot color map is not complete.
    	return Color.WHITE;
    }
}
