package ucot.ui.gui.dot;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Stack;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.core.ControlInterface;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;
import ucot.model.object.ObjectAnalyzeModel;
import ucot.ui.gui.GraphicalUI;

/**
 * A dot panel class inherited from JPanel class which is used for
 * drawing dot graphs from the given analyze model. This class
 * implements the observer interface which allows objects to keep
 * track on the current status of this panel.
 * 
 * This panel also observes the given analyze model and updates
 * the graph when the model changes.
 * 
 * There is also a method which allows highlighting any submodels
 * from the given analyze model.
 * 
 * @author ilanliuk, tujupien.
 */
public class DotPanel extends JPanel implements Observer {
	
	/**
	 * This class is a container for a dot job which includes all
	 * required information for rendering both highlighted and
	 * regular dot graphs. A container like this is required when
	 * dot execution is done in threads.
	 */
	private class DotJob {
		private AnalyzeModel analyzeModel = new ObjectAnalyzeModel();
		private AnalyzeModel highlightModel = new ObjectAnalyzeModel();
		
		private Set<String> entitiesToBeHighlighted =
	    	new HashSet<String>();
		private Set<String> entitiesInCurrentModel = 
	    	
	    	new HashSet<String>();
		private Set<String> entitiesRequestedForHighlighting = 
	    	new HashSet<String>();
		private Set<Vector<String>> parentsToBeHighlighted = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> parentsInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> parentsRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsToBeHighlighted = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> methodsRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesToBeHighlighted =
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesInCurrentModel = 
	    	new HashSet<Vector<String>>();
		private Set<Vector<String>> attributesRequestedForHighlighting = 
	    	new HashSet<Vector<String>>();
	}
	
	public static final long serialVersionUID = 3899637467435823526L;
	private static Stack<DotJob> dotJobStack = new Stack<DotJob>();
	private static boolean threadRunning = true;
	private static Thread executing = null;

	//TODO comment me
	public enum ExportImageType {PNG,JPG,EPS,PDF};
    
    protected long timeElapsed;
    protected JLabel timerComponent;
    
    //private AnalyzeModel analyzeModel;
    
    
    public DotColorTheme colorTheme = new DotColorTheme();
    public final static int SCROLL_SPEED = 20;
	
    private BufferedImage modelImage = null;
    protected GraphicalUI owner;
    
    private boolean horizontalLayout = true;
    private boolean isUpdating;
    
    /**
     * Constructor for DotPanel. This constructor adds an observer
     * to the analyze model and sets its own status to updating.
     * @param analyzeModel analyzeModel to use with this DotPanel.
     */
    public DotPanel(GraphicalUI owner) {
    	this.owner = owner;
        owner.getControlInterface().getAnalyzeModel().addObserver(this);
        this.isUpdating = true;
    }
    
    /**
     * Sets wether or not the dot should use the horizontal layout
     * for the entities and their relationships.
     * @param horizontalLayout New value for horizontal layout,
     *   true means yay for horizontal layouting.
     */
    public void setHorizontalLayout(boolean horizontalLayout) {
    	this.horizontalLayout = horizontalLayout;
    	highlight(null);
    }
    
    /**
     * Returns the current value of horizontal layouting.
     * @return True if horizontal layout is in use, else false.
     */
    public boolean getHorizontalLayout() {
    	return this.horizontalLayout;
    }

    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable observableObject, Object updationArg) {
        if ( !(updationArg instanceof Updation) )
            return;
        
        // Change updation status.
        Updation updation = (Updation) updationArg;
        if ( updation.getType() == UpdationType.MODIFICATION_STARTED ) {
            this.isUpdating = false;
        }
        if ( updation.getType() == UpdationType.READY ||
        		updation.getType() == UpdationType.CLEAR ) {
            this.isUpdating = true;
        }
        
        // Update graph if required.
        if ( this.isUpdating ) {
            this.updateModel(owner.getControlInterface().getAnalyzeModel());
        }
    }

    /**
     * Saves current image to given file in given format.
     * @param target Target file.
     * @param imageFormat Image format to use.
     */
    public void exportImage(File target, ExportImageType imageFormat) {
        
        // if no usecases have been added into 
        // analyzemodel modelImage is null..
        if ( modelImage == null || target == null ) {
            return; 
        }
        
        //Png and jpg exports are done in the same way
        if ( imageFormat == ExportImageType.PNG || imageFormat == ExportImageType.JPG) {
            try {
            	//Just tell ImageIO to write them to a file using a type given by imageTypeToString
                ImageIO.write(modelImage, imageTypeToString(imageFormat), target);
            } catch (IOException ex){
                ex.printStackTrace();
                throw new Error("Exporting image failed!");
            }            
        }
        
        //If we want eps, we have a spesific method for it
        if (imageFormat == ExportImageType.EPS) {
        	exportEps(target);
        }
        
        
        //Pdf export is done by doing eps export first and then converting eps to pdf
        //Using epstopdf program
        if (imageFormat == ExportImageType.PDF) {        	
            File epsFile = new File(this.owner.getProperties().
            		getProperty(GraphicalUI.PROPERTY_TEMP_EPS_FILE));
            
        	if (exportEps(epsFile)) 
        		epsToPdf(epsFile,target);
        	epsFile.delete();
        }
    }
    
    /**
     * Saves current image to given file in format
     * that is tried to guess from filename. 
     * @param target File pointing the saving destination.
     */
    public void exportImage(File target) {
        
        // Strip extension from filename
        String filename = target.getName();
        String extension = filename.substring(filename.lastIndexOf(".") + 1);
        boolean imagetypeFound = false;
        
        // try to resolve in wich imagetype we should save the image.
        for (ExportImageType imagetype : ExportImageType.values()) {
            if (extension.equals(imagetype.toString().toLowerCase())) {
                exportImage(target, imagetype);
                imagetypeFound = true;
                continue;
            }
        }
        if (!imagetypeFound) {
            throw new Error("No ExportImageType found for file extension of \"" + target.getName() + "\".");
        }
    }

    /**
     * Converts given imageformat to string
     * @param imageFormat imageformat to convert
     * @return png by default, othervise a format name
     */
    private String imageTypeToString(ExportImageType imageFormat) {
    	if (imageFormat == ExportImageType.EPS)
    		return "eps";
    	if (imageFormat == ExportImageType.JPG)
    		return "jpg";
    	if (imageFormat == ExportImageType.PDF)
    		return "pdf";
    	
    	//return png by default
    	return "png";
    }
    
    /**
     * Exports model to eps file
     * @param target file to save to
     * @return true if everything went ok
     */
    private boolean exportEps(File target)
    {
    	DotJob job = new DotJob(); 
    	job.analyzeModel.getEditor().merge(owner.getControlInterface().getAnalyzeModel());
    	File dotFile = createDotFile(job);
    	 // Check if we can find the dot executable.
        File dotExecutable = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_DOT_PATH));    
        
        
        //Check that dot executable exists
        if (!dotExecutable.exists()) {
            System.err.println("Error in DotPanel.java: can't find the dot.exe in given path '"
                            + dotExecutable.getAbsolutePath() + "'");
            dotFile.delete();
            return false;
        }

        

        // TODO Debug this on Linux.
        //Create a command for dot to generate eps file
        String executeCommand = dotExecutable.getAbsolutePath() + 
        	" -Gsize=8,10.7 -Gmargin=0 -Nfontsize=7 -Efontsize=6 -Gpagedir=TL -Gremincross=true -Gconcentrate=true -Tps2 -o" + 
        	target.getAbsolutePath() + " " + dotFile.getAbsolutePath();
        

        // Run dot executable.
        Process dotProcess = null;
        
        try { //try executing the command
            dotProcess = Runtime.getRuntime().exec(executeCommand);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            dotFile.delete();
            return false;
        }

        //Sanity check
        if (dotProcess == null) {
        	dotFile.delete();
            return false;
        }

        try { 
            dotProcess.waitFor();
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
            dotFile.delete();
            return false;
        }
        
        return true;
    }
    
    /**
     * Converts eps file to pdf
     * @param epsFile eps file to read
     * @param pdfFile pdf to save
     * @return true if everything went ok
     */
    private boolean epsToPdf(File epsFile, File pdfFile)
    { 
    	File epstopdfExecutable = new File(this.owner.getProperties().
    			getProperty(GraphicalUI.PROPERTY_EPS_TO_PDF_PATH));
    	String executeCommand = epstopdfExecutable.getAbsolutePath() + " --outfile=" + pdfFile.getAbsolutePath() + " " + epsFile.getAbsolutePath();
    	
    	//Run epstopdf executable.
        Process epstopdfProcess = null;
        try {
        	epstopdfProcess = Runtime.getRuntime().exec(executeCommand);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            return false;
        }

        if (epstopdfProcess == null)
            return false;

        try {
        	epstopdfProcess.waitFor();
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
            return false;
        }
        
        return true;
    }   

    
    /**
     * This method runs the dot executable with the given dot input file
     * and returns the location of the image file.
     * @param dotInputFile Input file for dot executable.
     * @return File where the image is located. Null is returned if
     *   something went wrog.
     */
    private File runDot(File dotInputFile) {
        // Ensure that we can find the dot executable.
        File dotExecutable = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_DOT_PATH));
        if ( !dotExecutable.exists() ) {
            System.err.println("Error in DotPanel.java: can't find the dot.exe in given path '"
                            + dotExecutable.getAbsolutePath() + "'");
            return null;
        }

        File imageFile = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_TEMP_OUTPUT_FILE));

        // TODO Debug this on Linux.
        // Get the output from dot executable as a PNG image.
        String executeCommand = dotExecutable.getAbsolutePath() + " -Tpng "
                + dotInputFile.getAbsolutePath() + " -o "
                + imageFile.getAbsolutePath();

        // Run dot executable.
        Process dotProcess = null;
        try {
            dotProcess = Runtime.getRuntime().exec(executeCommand);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            imageFile = null;
        }

        if (dotProcess == null)
            return null;

        try {
            dotProcess.waitFor();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        return imageFile;
    }
    
    /**
     * Method for writing the methods of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose methods are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityMethods(DotJob job, BufferedWriter writer,
    		String entity) throws IOException {
    	ModelEditor editor = job.analyzeModel.getEditor();
    	Set<String> methods = editor.getMethodNames(entity);
        
        try {
	        for ( String method : methods ) {
	            Set<String> targets = editor.getEntitiesInfluencedByMethod(
	                    entity, method);
	
	            for ( String target : targets ) {
	                writer.write("  \"" + entity + "\" -> \"" + target
	                        + "\" [label=\"" + method
	                        + "\", arrowhead=\"vee\", ");
	                Vector<String> compare = new Vector<String>();
	                compare.add(entity);
	                compare.add(method);
	                compare.add(target);
	                writer.write(colorTheme.getColorString(job.methodsToBeHighlighted
	                		!= null && job.methodsToBeHighlighted.contains(compare)));
	                writer.write("];");
	                writer.newLine();
	            }
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Method for writing the attributes of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose attributes are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityAttributes(DotJob job, BufferedWriter writer,
    		String entity) throws IOException {
    	ModelEditor editor = job.analyzeModel.getEditor();
        Set<String> attributes = editor.getAttributeNames(entity);
        for ( String attribute : attributes ) {
        	writer.write("  \"" + entity + "\" -> \"" + attribute +
        			"\" [taillabel=\""+editor.getAttributeFromCardinal(entity, attribute)
        			+"\", headlabel=\""+editor.getAttributeToCardinal(entity, attribute)+
        			"\", arrowtail=\"diamond\", arrowsize=\"1.3\", labeldistance=\"1.7\""+
        			", arrowhead=\"none\", ");
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(attribute);
        	writer.write(colorTheme.getColorString(job.attributesToBeHighlighted
        			!= null && job.attributesToBeHighlighted.contains(compare)));
        	writer.write("];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing the parents of one entity to the dot file.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose parents are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityParents(DotJob job,	BufferedWriter writer,
    		String entity) throws IOException {
    	Set<String> parents = job.analyzeModel.getEditor().getParents(entity);
        for ( String parent : parents ) {
        	writer.write("  \"" + entity + "\" -> \"" + parent + 
        			"\" [arrowhead=\"onormal\", arrowsize=\"1.4\", ");
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(parent);
        	writer.write(colorTheme.getColorString(job.parentsToBeHighlighted
        			!= null && job.parentsToBeHighlighted.contains(compare)));
        	writer.write("];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing all entities to the dot file with all their
     * attribute, method and parent relationships.
     * @param job Defines the entities to be written.
     * @param writer BufferedWriter to use for writing.
     * @throws IOException An exception is thrown if something goes wrong.
     */
    private void writeEntities(DotJob job, BufferedWriter writer) throws IOException {
        Set<String> entityNames = job.analyzeModel.getEditor().getEntityNames();
        
        try {
	    	for ( String entity : entityNames ) {
	        	
	            writer.write("  \"" + entity + "\" [");
	            writer.write(colorTheme.getColorString(job.entitiesToBeHighlighted
	            		!= null && job.entitiesToBeHighlighted.contains(entity)));
	            
	            writer.write("];");
	            writer.newLine();
	            
	            writeEntityMethods(job, writer, entity);
	            writeEntityAttributes(job, writer, entity);
	            writeEntityParents(job, writer, entity);
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Writes the analyze model to an external file in dot syntax.
     * @param job Dot job which defines the model to be drawn.
     * @return File containing the analyze model in dot syntax.
     *   Null is returned if something went wrong.
     */
    private File createDotFile(DotJob job) {

        File result = new File(this.owner.getProperties().
        		getProperty(GraphicalUI.PROPERTY_TEMP_INPUT_FILE));

        // Initialize stream to file for writing.
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(result));

            // Write starting tag into file and graph-wide attributes.
            writer.write("digraph G { [" + colorTheme.getBackgroundColorString() + 
            		", rankdir=\"" + (horizontalLayout ? "TB" : "LR" ) + "\"" +
            		"]");
            writer.newLine();
            
            // Write the entities and their relationships to the dot file.
            writeEntities(job, writer);

            // Write end tag into file.
            writer.write("}");
            writer.newLine();

            writer.flush();
            writer.close();

        } catch (IOException e) {
            System.err.println(e.getMessage());
            return null;
        }

        return result;
    }
    
    /**
     * Method for mapping the current analyze model. This is required to
     * figure out differences with the submodel that is required to be
     * highlighted.
     * @param job Dot job where mapping will be done.
     */
    private void mapCurrentModel(DotJob job) {
    	for ( String entity : job.entitiesInCurrentModel ) {
    		for ( String method : job.analyzeModel.getEditor().getMethodNames(entity) ) {
    			for ( String target : job.analyzeModel.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				job.methodsInCurrentModel.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : job.analyzeModel.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			job.attributesInCurrentModel.add(attributeVector);
    		}
    		
    		for ( String parent : job.analyzeModel.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			job.parentsInCurrentModel.add(parentVector);
    		}
    	}
    }
    
    /**
     * Method for mapping the submodel for highlight request.
     * @param job Dot job to be handled.
     * @param drawNewElements Indicates wether or not the new elements are
     *   supposed to be drawn in the highlighted model. New elements are
     *   those that exist in the highlight request but do not exist in the
     *   current analyze model.
     */
    private void mapHighlightRequest(DotJob job, boolean drawNewElements) {
    	for ( String entity : job.entitiesRequestedForHighlighting ) {
    		for ( String method: job.highlightModel.getEditor().getMethodNames(entity) ) {
    			for ( String target: job.highlightModel.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				job.methodsRequestedForHighlighting.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : job.highlightModel.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			job.attributesRequestedForHighlighting.add(attributeVector);
    		}
    		
    		for ( String parent : job.highlightModel.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			job.parentsRequestedForHighlighting.add(parentVector);
    		}
    		
    		if ( !job.entitiesInCurrentModel.contains(entity) && !drawNewElements ) continue;
			job.entitiesToBeHighlighted.add(entity);
    	}
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * @param highlight The submodel to be highlighted. If this argument is
     *   null, then all applied highlights are removed.
     * @param drawNewElements Indicates wether or not those elements
     *   which do not exist in the current analyze model should be
     *   also drawn and highlighted.
     */
    public void highlight(AnalyzeModel highlight, boolean drawNewElements) {
    	DotJob job = new DotJob();
    	job.analyzeModel.getEditor().merge(owner.getControlInterface().getAnalyzeModel());
    	job.highlightModel = highlight;
    	
    	// Initialize sets for current entities and their relationships
    	// and then map those entities and their relationships which are
    	// requested to be highlighted.
    	job.entitiesInCurrentModel = owner.getControlInterface().getAnalyzeModel().getEditor().getEntityNames();
    	if ( highlight != null ) job.entitiesRequestedForHighlighting =
    		highlight.getEditor().getEntityNames();
    	
    	// If highlight is null, then just remove any highlightings that
    	// are currently applied.
    	if ( highlight != null ) {
        	
        	mapCurrentModel(job);
        	
        	mapHighlightRequest(job, drawNewElements);
        	
        	// Calculate methods to be highlighted.
	    	for ( Vector<String> method : job.methodsRequestedForHighlighting ) {
	    		if ( !job.methodsInCurrentModel.contains(method) &&
	    				!drawNewElements ) continue;
	    		job.methodsToBeHighlighted.add(method);
	    	}
	    	
	    	// Calculate attributes to be highlighted.
	    	for ( Vector<String> attribute : job.attributesRequestedForHighlighting ) {
	    		if ( !job.attributesInCurrentModel.contains(attribute) &&
	    				!drawNewElements ) continue;
	    		job.attributesToBeHighlighted.add(attribute);
	    	}
	    	
	    	// Calculate parents to be highlighted.
	    	for ( Vector<String> parent : job.parentsRequestedForHighlighting ) {
	    		if ( !job.parentsInCurrentModel.contains(parent) &&
	    				!drawNewElements ) continue;
	    		job.parentsToBeHighlighted.add(parent);
	    	}
    	}
    	
    	updateModel(job);
    	
    	// Destroy class sets associated with highlighting.
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * As default we assume that new entities don't need to be drawn.
     * @param highlight Submodel to be highlighted.
     */
    public void highlight(AnalyzeModel highlight) {
    	highlight(highlight, false);
    }
    
    /**
     * Method for updating the model. This method pushes
     * the given dot job to the top of the dot job stack
     * and notifies the running thread about it. If no
     * updation thread is running yet, one is created.
     * @param job Dot job to be run next.
     */
    private void updateModel(final DotJob job) {
    	synchronized (dotJobStack) {
    		dotJobStack.push(job);
    	}
    	if ( executing == null ) {
    		executing = new Thread(new Runnable() {
				public void run() {
					executeDot();
				}        	
	        });
    		executing.start();
    	} else {
    		synchronized (executing) {
    			executing.notify();
    		}
    	}
    }
    
    /**
     * Method for updating the analyze model.
     * @param analyzeModel New analyze model.
     */
    public void updateModel(AnalyzeModel analyzeModel) {
    	DotJob job = new DotJob();
    	//this.analyzeModel = analyzeModel;
    	job.analyzeModel.getEditor().merge(analyzeModel);
    	updateModel(job);
    }
    
    /**
     * This finalize method makes sure the thread
     * running for panel update shuts down.
     */
    protected void finalize() throws Throwable {
    	threadRunning = false;
    	synchronized (executing) {
    		executing.notify();
    	}
    	super.finalize();
    }
    
    /**
     * This method draws the graph generated by dot to the panel's
     * canvas.
     */
    public void paintComponent(Graphics g) {
    	super.setBackground(colorTheme.getBackgroundColorAsJavaObject());
        super.paintComponent(g);
        int x = 0, y = 0;
        // Update component size to match the scroller size.
        if (this.modelImage != null) {
            x = Math.max(0, (int) Math.floor((this.getWidth() - this.modelImage
                    .getWidth()) / 2));
            y = Math.max(0,
                    (int) Math.floor((this.getHeight() - this.modelImage
                            .getHeight()) / 2));

            g.drawImage(this.modelImage, x, y, this);
            
            this.setPreferredSize(new Dimension(this.modelImage.getWidth(), this.modelImage.getHeight()));
            this.revalidate();
        }
    }
    
    /**
     * This method should be executed in its own thread. This thread keeps
     * running until threadRunning class variable is set to false.
     * It takes the latest job from the dot job stack and disposes all the
     * other jobs in the stack at the same time. The newest job is then
     * modeled and drawn to the canvas. After that the thread sleeps until
     * waken again by notification [executing.notify()].
     */
	private void executeDot() {
		
		while ( threadRunning ) {
			if ( !dotJobStack.isEmpty() ) {
				
				long startTime = System.currentTimeMillis();
				DotJob job = null;
				
				synchronized (dotJobStack) {
					job = dotJobStack.pop();
					dotJobStack.clear();
				}
				
				File dotFile = createDotFile(job);
		        if (dotFile == null) {
		            System.err
		                    .println("Error in initializing dotFile, exiting updateAnalyzemodel.");
		            return;
		        }
		
		        File imageFile = runDot(dotFile);
		        if (imageFile == null) {
		            System.err
		                    .println("Error in initializing imageFile, exiting updateAnalyzemodel.");
		            return;
		        }
		        
		        // Check if image file exists... just to be sure.
		        if (!imageFile.exists()) {
		            System.err.println("DotPanel.java: imageFile (" + imageFile.getName() + ") not found.");
		            return;
		        }
		
		        // If everything went ok, load image from file.
		        try {
		            this.modelImage = ImageIO.read(imageFile);
		        } catch (IOException e) {
		            System.err.println("DotPanel.java: Error in reading image from file.");
		            e.printStackTrace();
		            return;
		        }
		
		        // Remove temp files
		        //dotFile.delete();
		        //imageFile.delete();
		        
		        this.repaint();
		        
		        // Uncomment the next line of code to confirm that the dot panel really
		        // is running in a separate thread.
		        // for ( int i = 0; i < 100000; System.out.println(++i) );
		        
		        timeElapsed = System.currentTimeMillis() - startTime;
		        if ( timerComponent != null ) {
		        	timerComponent.setText( " "+timeElapsed+" ms " );
		        }
			} else {
				try {
					synchronized (executing) {
						executing.wait();
					}
				} catch (Exception e) { e.printStackTrace(); }
			}
		}
	}
	
	public void setTimerComponent(JLabel timerComponent) {
		this.timerComponent = timerComponent;
	}	
}
